#' Draws hatching or crosshatching over a mask on a map
#' 
#' Adds hatching or crosshatching lines over grid cells of a map based on a 
#' logical mask. The hatching is applied only to grid cells where the mask is
#' TRUE (or 1). A number of graphical options are available to customize the appearance
#' of the hatching, such as line density, angle, line width, and color.
#' Optionally, crosshatching can be drawn using a second set of lines in the
#' opposite direction.
#' The function assumes that the input mask is a 2D array with dimensions
#' corresponding to latitude (rows) and longitude (columns), and it is designed
#' to be used as a layer over existing map plots (e.g., within a call to 
#' \code{VizEquiMap}) and does not initiate a plot by itself.
#'
#'@param mask Logical or binary (0/1) array with two named dimensions:
#' c(latitude, longitude).Hatching is applied to grid cells where 'mask' is
#' TRUE (or 1). Arrays with dimensions c(longitude, latitude) are also accepted,
#' but the resulting hatching may appear transposed. To ensure correct alignment
#' with the map, provide 'data'. The function will compare the dimension order
#' of 'mask' and 'data', and automatically transpose 'mask' if the latitude and
#' longitude dimensions appear to be reversed.
#'@param lat Numeric vector of latitude locations of the cell centers of the 
#'  grid.
#'@param lon Numeric vector of longitude locations of the cell centers of the 
#'  grid.
#'@param data Array of the data that the hatching will be drawn over. The array 
#'  should have named latidude and longitude dimensions. If the dimension order
#'  is reversed relative to 'mask', the mask is automatically transposed.
#'  Default is NULL.
#'@param density The density of shading lines, in lines per inch. A zero value of
#'  density means no shading nor filling, whereas negative values and NA
#'  suppress shading (and so allow color filling). NULL means that no shading
#'  lines are drawn. Default is 10.
#'@param angle The slope of shading lines, given as an angle in degrees
#'  (counter-clockwise). Default is 45.
#'@param col_line Color of the hatching lines. Default is \code{"#252525"}.
#'@param lwd_size The line width, a positive number. The interpretation is
#'  device-specific, and some devices do not implement line widths less than
#'  one. Default is 0.5.
#'@param crosshatching A logical value indicating crosshatching. If TRUE, adds a
#'  second set of lines in the opposite angle. Default is FALSE.
#'@examples
#'  \dontrun{
#'mask <- array(c(TRUE, FALSE, TRUE, FALSE), dim = c(lat = 2, lon = 2))
#'lats <- 1:2
#'lons <- 1:2
#'image(lon_small, lat_small, matrix(1:4, nrow = 2))
#'Hatching(mask = mask_small, lat = lat_small, lon = lon_small, lwd_size = 2)
#'  }
#'@importFrom s2dv InsertDim
#'@export

Hatching <- function(mask, lat, lon, data = NULL, density = 10, angle = 45,
                     col_line = '#252525', lwd_size = 0.5,
                     crosshatching = FALSE) {
  
  if (length(dim(mask)) != 2 || is.null(names(dim(mask)))) {
    stop("The hatching 'mask' array should have two named latitude and longitude dimensions")
  } 
  if (!(any(names(dim(mask)) %in% .KnownLonNames()) &&
        any(names(dim(mask)) %in% .KnownLatNames()))) {
    stop("Dimension names of the hatching 'mask' doesn't correspond to any
          coordinates names supported by s2dv package.")
  } 
  lon_dim <- names(dim(mask))[names(dim(mask)) %in% .KnownLonNames()]
  lat_dim <- names(dim(mask))[names(dim(mask)) %in% .KnownLatNames()]
  if (dim(mask)[[lat_dim]] != length(lat) | dim(mask)[[lon_dim]] != length(lon)) {
    stop("The dimensions of the hatching 'mask' do not match the lengths of 'lat' and 'lon'.")
  }
  
  # If 'mask' dimensions are reversed respect 'data', reorder them
  if (!is.null(data)) {
    if (!is.null(names(dim(data)))) {
      if (!identical(names(dim(mask)), names(dim(data)))) {
        mask <- aperm(mask, match(names(dim(data)), names(dim(mask))))
      }
    }
  }
  
  # Convert numeric values to logical values in mask
  if (is.numeric(mask)) {
    if (all(mask %in% c(0, 1))) {
      mask <- mask == 1
    } else {
      stop("The hatching 'mask' array must have only TRUE/FALSE or 0/1.")
    }
  } else if (!is.logical(mask)) {
    stop("The hatching 'mask' array must have only TRUE/FALSE or 0/1.")
  }
  
  # Check lon, lat
  if (!is.numeric(lon) || !is.numeric(lat)) {
    stop("Parameters 'lon' and 'lat' must be numeric vectors.")
  }
  
  # Check data
  if (!is.null(data)) {
    if (!is.array(data)) {
      stop("Parameter 'data' must be a numeric array.")
    }
  }
  
  # Check density
  if (!is.null(density)) {
    if (!is.numeric(density) || length(density) != 1) {
      stop("Parameter 'density' must be a single numeric value.")
    }
  }
  
  # Check angle
  if (!is.null(angle)) {
    if (!is.numeric(angle) || length(angle) != 1) {
      stop("Parameter 'angle' must be a single numeric value.")
    }
  }
  
  # Check col_line
  if (!is.null(col_line)) {
    if (!.IsColor(col_line)) {
      stop("Parameter 'col_line' must be a valid colour identifier.")
    }
  }
  
  # Check lwd_size
  if (!is.null(lwd_size)) {
    if (!is.numeric(lwd_size) || length(lwd_size) != 1 || lwd_size <= 0) {
      stop("Parameter 'lwd_size' must be a single positive numeric value.")
    }
  }
  
  # Check crosshatching
  if (!is.null(crosshatching)) {
    if (!is.logical(crosshatching) || length(crosshatching) != 1) {
      stop("Parameter 'crosshatching' must be a single logical value (TRUE or FALSE).")
    }
  }
  
  # Helper function to calculate cell edges (breaks between lat/lon points)
  get_edges <- function(vec) {
    mid <- diff(vec) / 2
    edges <- c(vec[1] - mid[1], vec[-length(vec)] + mid, vec[length(vec)] + mid[length(mid)])
    return(edges)
  }
  # Find lon and lat edges
  x_edges <- get_edges(lon)
  y_edges <- get_edges(lat)
  # Loop through mask
  for (i in seq_len(nrow(mask))) {
    for (j in seq_len(ncol(mask))) {
      if (mask[i, j]) {
        # Calculate cell coordinates
        x_min <- x_edges[i]
        x_max <- x_edges[i + 1]
        y_min <- y_edges[j]
        y_max <- y_edges[j + 1]
        # Draw grid cell
        xs <- c(x_min, x_min, x_max, x_max)
        ys <- c(y_min, y_max, y_max, y_min)
        # Fill with diagonal lines
        polygon(xs, ys, density = density, angle = angle, border = NA, lwd = lwd_size, col = col_line)
        if (crosshatching) {
          polygon(xs, ys, density = density, angle = -angle, border = NA, lwd = lwd_size, col = col_line)
        }
      }
    }
  }
}

